%{
  Calculation of the hydrostatic pressure resultant and
  momentum acting on a submerged triangulated surface.
   
  [ResultantF,ResultantM] = hydrostatics('porsche_scaled.stl',[0 0 0],101325,1000,[0 0 -9.81],[0 0 0]);
  [ResultantF,ResultantM] = hydrostatics('verticalBulkhead.stl',[0 0 0],0.0,1000,[0 0 9.81],[0 0 0]);
  [ResultantF,ResultantM] = hydrostatics('tetrahedron.stl',[0.75 0.5 0],0.0,1000,[0 0 9.81],[0 0 0]);

  INPUT PARAMETERS:
  * stl........................character string; filename of stl file containing surface mesh
  * Pole.......................double, 1X3 array; cartesian coordinates of pole for torque calculation
  * P0.........................double, scalar; atmospheric pressure on free surface
  * rho........................double, scalar; fluid's density
  * g..........................double, 1X3 array; gravitational acceleration vector
  * x0.........................double, 1X3 array; cartesian coordinates of point on free surface

  OUTPUT VARIABLES:
  * ResultantF.................
  * ResultantM.................
  * T..........................
  * tri........................
  * vert.......................
  * edges......................double, NumEdges X 2; array with index of first- and end-point of each mesh's edge
  * edgel......................double, NumEdges X 2; array with indexes of incident triangles ("0" means "no incident triangle").
                                                     

%}
function [ResultantF,ResultantM,T,tri,vert,edges,edgel] = hydrostatics(stl,Pole,P0,rho,g,x0)

% Modulus of gravitational acceleration
grav = norm(g);

% Load triangulated surface
[tri,vert] = stlread(stl);

% Nr of triangular elements
Ne = size(tri,1);
% Nr of vertex nodes (with repetitions)
Nn = size(vert,1);

% "Unify" vertices
[vert,I,J] = unique(vert,'rows');
tri = J(tri);

Nn = size(vert,1);

% Define edges
edges = [tri(:,1),tri(:,2);tri(:,2),tri(:,3);tri(:,3),tri(:,1)];
Edges = sort(edges(:,1:2),2);
[Edges,I,J] = unique(Edges,'rows');

edges = edges(I,1:2);

% Associate each edge with the connected triangles
Ned = size(edges,1);
edgel = zeros(Ned,2);
for j=1:3*Ne
    if edgel(J(j),1)==0
        iloc = 1;
    else
        iloc = 2;
    end;
    edgel(J(j),iloc) = j-floor((j-1)/Ne)*Ne;
end;

% Is geometry closed?
isclosed = (min(edgel(:,2))>0);

% Plot geometry with immersion
immersion = sum((vert-repmat(x0(:)',[Nn,1])).*repmat(g(:)',[Nn,1]),2)./grav;
hfig=figure;
trimesh(tri,vert(:,1),vert(:,2),vert(:,3),immersion,'FaceColor','interp','edgecolor','k');
axis equal;
colormap cool
lighting phong
colorbar

% Centroid of each triangle
Centroid = (vert(tri(:,1),:)+vert(tri(:,2),:)+vert(tri(:,3),:))./3;

% Area of each triangle
e12 = vert(tri(:,2),:)-vert(tri(:,1),:);
e13 = vert(tri(:,3),:)-vert(tri(:,1),:);
Area = 0.5*cross(e12,e13,2);

% Volume of closed triangulated suface, using Gauss' theorem
if isclosed
    Volume = sum(sum(Centroid.*Area,2),1)/3;
    Buoyancy = rho*grav*Volume;
end;


% Pressure at nodes
pressure = P0 + rho*((vert-repmat(x0,[Nn,1]))*g(:));

% Pressure at triangle's centroid
pressureC = P0 + rho*((Centroid-repmat(x0,[Ne,1]))*g(:));


% Plot geometry with pressure distribution
hfig2=figure;
title('Pressure distribution [kPa]')
trimesh(tri,vert(:,1),vert(:,2),vert(:,3),pressure./1e3,'FaceColor','interp','edgecolor','k');
axis equal;
colormap cool
lighting phong
colorbar



% Force on each triangle
Force = -(1/3)*repmat((pressure(tri(:,1))+pressure(tri(:,2))+pressure(tri(:,3))),[1,3]).*Area;

% Alternative calculation (should be equivalent for linear pressure
% distribution)
ForceC = -repmat(pressureC,[1,3]).*Area;

% Resultant
ResultantF = sum(Force,1);

% Resultant (alternative method)
ResultantFC = sum(ForceC,1);

% Compare with buoyancy force obtained by divergence theorem
if isclosed
    disp(['Discrepancy on buoyancy force: ',num2str(100*(norm(ResultantF)-Buoyancy)/Buoyancy)])
end;

% Calculate momentum of pressure forces for each triangle,
Momentum = zeros(Ne,3);
B        = [2 1 1;1 2 1;1 1 2];
for e=1:Ne
    
    % Vector distance from Pole
    xx = vert(tri(e,:),:)-repmat(Pole,[3,1]);
    
    % Pressure at triangle's nodes
    pe = pressure(tri(e,:));
    
    % Calculate momentum exerted by pressure force acting on a single
    % triangle
    Momentum(e,:) = 0;
    for j=1:3
        Me = 0;
        for k=1:3
            Me = Me - B(j,k)*cross(xx(k,:),Area(e,:));
        end;
        Momentum(e,:) = Momentum(e,:) + Me.*pe(j)./12;
    end;
    
end;

% Sum over all triangles
ResultantM = sum(Momentum,1);

% Invariant trinomial
T = sum(ResultantM(:).*ResultantF(:));


% Central axis (parallel to force resultant)
RM0 = cross(ResultantF(:),ResultantM(:));
jax = -cross(ResultantF(:),RM0); jax = jax./norm(jax);
Center = Pole(:) + (RM0./norm(RM0)).*(dot(ResultantM(:),jax)/norm(ResultantF));


sR = ResultantF(:)./norm(ResultantF);
P1 = Center-[0;0;10];
P2 = Center+[0;0;10];
figure(hfig2); hold on;
h=plot3([P1(1) P2(1)],[P1(2) P2(2)],[P1(3) P2(3)],'k-');
set(h,'linewidth',5.0);
alpha(0.5)

return;

end